import { invoke } from '@tauri-apps/api/core';
import { atom, useAtomValue, useSetAtom } from 'jotai';
import { atomWithRefresh } from 'jotai/utils';
import { useCallback } from 'react';
import { NotificationType, ToastDuration, useNotification } from '../components/Notifications';
import { Pattern, Pulse, totalDuration } from './pattern-model';

export const SearchKeywordAtom = atom<string | null>(null);
export const PatternsAtom = atomWithRefresh(async (get) => {
  try {
    const keyword = get(SearchKeywordAtom);
    const patterns = await invoke<Pattern[]>('list_patterns', { keyword });
    return patterns;
  } catch (e) {
    console.error('[retrieving pattern list]', e);
  }
  return [];
});
export const SelectedPatternIdAtom = atom<string | null>(null);
export const CurrentPatternAtom = atomWithRefresh<Pattern | null>(async (get) => {
  try {
    const patternId = get(SelectedPatternIdAtom);
    if (patternId === null) {
      return null;
    }
    const pattern = await invoke('get_pattern', { patternId });
    return pattern;
  } catch (e) {
    console.error('[retrieving pattern]', e);
  }
  return null;
});
export const CurrentPatternDuration = atom((get) => {
  const currentPattern = get(CurrentPatternAtom);
  if (!currentPattern) return 0;
  return totalDuration(currentPattern);
});
export const SelectedPulseIdAtom = atom<string | null>(null);
export const SelectedPulseAtom = atom<Pulse | null>(async (get) => {
  const pattern = await get(CurrentPatternAtom);
  const selectedPulseId = get(SelectedPulseIdAtom);
  console.debug('[refresh selected pulse]', selectedPulseId, pattern);
  return pattern?.pulses?.find((pulse) => pulse.id === selectedPulseId) ?? null;
});

export function useSavePattern() {
  const refreshPatterns = useSetAtom(PatternsAtom);
  const selectedPatternId = useAtomValue(SelectedPatternIdAtom);
  const { showToast } = useNotification();

  const savePattern = useCallback(
    async (pattern: Pattern) => {
      try {
        await invoke('save_pattern', { pattern });
        refreshPatterns();
        return true;
      } catch (error) {
        console.error('[save pattern]', error);
        showToast(
          NotificationType.ERROR,
          'Failed to save pattern. Please try again.',
          'material-symbols-light:error-outline',
          ToastDuration.MEDIUM,
        );
      }
      return false;
    },
    [selectedPatternId],
  );

  return savePattern;
}
